/*
 *  Copyright (C) Chaperon. All rights reserved.                               
 *  ------------------------------------------------------------------------- 
 *  This software is published under the terms of the Apache Software License 
 *  version 1.1, a copy of which has been included  with this distribution in 
 *  the LICENSE file.                                                         
 */

package net.sourceforge.chaperon.grammar.token.definition;

import net.sourceforge.chaperon.grammar.Grammar;
import net.sourceforge.chaperon.helpers.IntegerList;
import net.sourceforge.chaperon.parser.RegexAutomate;

/**
 * This class describes a regex element for the definition
 * of a token
 *
 * @author Stephan Michels
 * @version $Revision: 1.2 $ $Date: 2002/05/06 16:22:56 $
 */
public abstract class DefinitionElement implements Cloneable
{

  /** minOccurs property */
  int _minOccurs = 1;

  /** minOccurs property */
  int _maxOccurs = 1;

  /**
   * Sets the minimum occurs property
   *
   * @param minOccurs Minimum occurs property
   */
  public void setMinOccurs(int minOccurs)
  {
    if (minOccurs >= 0)
      _minOccurs = minOccurs;
  }

  /**
   * Returns the minimum occurs property
   *
   * @return Minimum occurs property
   */
  public int getMinOccurs()
  {
    return _minOccurs;
  }

  /**
   * Sets the maximum occurs property
   *
   * @param maxOccurs Maximum occurs property
   */
  public void setMaxOccurs(int maxOccurs)
  {
    if (maxOccurs >= _minOccurs)
      _maxOccurs = maxOccurs;
  }

  /**
   * Returns the maximum occurs property
   *
   * @return Maximum occurs property
   */
  public int getMaxOccurs()
  {
    return _maxOccurs;
  }

  public abstract Object clone() throws CloneNotSupportedException;
}
